

function ItemListTreeView(pXulTree){
	this.fCh = [];
	this.rowCount = 0;
	this.fXulTree = pXulTree;

	this.fLastIdx = -1;
	this.fLastRow = null;
}


ItemListTreeView.prototype.buildTree = function(pKeyColumns, pPoolDatas){
	this.fKeyColumns = pKeyColumns;
	var vColumns = this.fXulTree.columns;

	for(var i = 0; i < vColumns.length; i++) {
		vColumns[i].element.compileCellBuilder(this);
	}

	this.fPoolDatas = pPoolDatas || [];
	this.fLastIdx = -1;
	this.fLastRow = null;
	this.fCh = [];

	var vPrimaryCol = vColumns.getPrimaryColumn();
	if(vPrimaryCol) {
		this.getCellBuiler(vPrimaryCol).buildTree(this, vPrimaryCol);
	} else {
		this.defaultBuildTree();
	}
	this.selection.select(-1);
	this.fXulTree.treeBoxObject.scrollToRow(0);
}

ItemListTreeView.prototype.cleanTree = function(){
	this.fPoolDatas = [];
	this.fLastIdx = -1;
	this.fLastRow = null;
	this.fCh = [];
	this.selection.select(-1);
	var vPrevCount = this.rowCount;
	this.rowCount = 0;
	var vTreeBo = this.fXulTree.treeBoxObject;
	vTreeBo.rowCountChanged(0, -vPrevCount);
	vTreeBo.scrollToRow(0);
}

ItemListTreeView.prototype.getIdxColumn = function(pKeyColumns){
	return this.fKeyColumns.indexOf(pKeyColumns);
}

ItemListTreeView.prototype.defaultBuildTree = function(){
	var vColumns = this.fXulTree.columns;

	var vSortedCol = vColumns.getSortedColumn();
	if(vSortedCol) this.getCellBuiler(vSortedCol).sortPoolDatas(this, vSortedCol);

	this.fCh = [];
	for(var i = 0; i < this.fPoolDatas.length; i++) {
		this.fCh.push(new ILRowLeaf(this, i, this.fPoolDatas[i]));
	}

	var vPrevCount = this.rowCount;
	this.rowCount = this.fCh.length;
	var vTreeBo = this.fXulTree.treeBoxObject;
	if(vPrevCount>0) vTreeBo.invalidateRange(0, vPrevCount-1);
	vTreeBo.rowCountChanged(vPrevCount, this.rowCount - vPrevCount);
}


ItemListTreeView.prototype.getPoolDatas = function(){
	return this.fPoolDatas;
}


ItemListTreeView.prototype.getCellBuiler= function(pCol){
	return pCol.element.fCellBuilder;
}



ItemListTreeView.prototype.setSortedColumn = function(pCol, pDirection){
	try {
	var vColumns = this.fXulTree.columns;
	var vPrimaryCol = vColumns.getPrimaryColumn();
	if(vPrimaryCol) vPrimaryCol.removeAttribute("primary");
	var vSortedCol = vColumns.getSortedColumn();
	if(vSortedCol && vSortedCol != pCol) vSortedCol.element.removeAttribute("sortDirection");
	pCol.element.setAttribute("sortDirection", pDirection);
	vColumns.invalidateColumns();
	this.defaultBuildTree();
	this.selection.select(-1);
	this.fXulTree.treeBoxObject.scrollToRow(0);
	}catch(e){
	ut.debug("ItemListTreeView.prototype.setSortedColumn:::"+e);
	}
}



ItemListTreeView.prototype.setTree = function(pTree){
}


ItemListTreeView.prototype.getCellText = function(pRow, pCol){
	return this.getCellBuiler(pCol).getCellText(this.getRowFromIndex(pRow), pCol);
}

ItemListTreeView.prototype.isSeparator = function(pRow){
	return false;
}

ItemListTreeView.prototype.isEditable = function(pRow, pCol){
	return false;
}

ItemListTreeView.prototype.isSelectable = function(pRow, pCol){
	return false;
}

ItemListTreeView.prototype.isSorted = function(){
	return false;
}

ItemListTreeView.prototype.getImageSrc = function(pRow, pCol){
	return this.getCellBuiler(pCol).getImageSrc(this.getRowFromIndex(pRow), pCol);
}

ItemListTreeView.prototype.getRowProperties = function(pRow, pProps){
	return this.getCellBuiler(pCol).getRowProperties(this.getRowFromIndex(pRow), pProps);
}

ItemListTreeView.prototype.getCellProperties = function(pRow, pCol, pProps){
	return this.getCellBuiler(pCol).getCellProperties(this.getRowFromIndex(pRow), pCol, pProps);
}

ItemListTreeView.prototype.getColumnProperties = function(pCol, pProps){
	return this.getCellBuiler(pCol).getColumnProperties(this, pCol, pProps);
}

ItemListTreeView.prototype.hasNextSibling = function(pRow, pAfterIndex){
	var vRow = this.getRowFromIndex(pRow);
	if(!vRow) return false;
	var vParent = vRow.fParent;
	var vIdxRowLastSibling = pRow;
	for(var i = vRow.fOffsetInParent, vLast = vParent.fCh.length-1; i < vLast; i++) {
		vIdxRowLastSibling += vParent.fCh[i].rowCount;
	}
	return (vIdxRowLastSibling > pAfterIndex);
}




ItemListTreeView.prototype.getLevel = function(pRow){
	var vRow = this.getRowFromIndex(pRow);
	if(!vRow) return 0;
	var vLevel = -1;
	while(vRow) {
		vLevel++;
		vRow = vRow.fParent;
	}
	return vLevel;
}
ItemListTreeView.prototype.getParentIndex = function(pRow){
	var vRow = this.getRowFromIndex(pRow);
	if(!vRow) return -1;
	return vRow.fParent.getIndex();
}
ItemListTreeView.prototype.isContainer = function(pRow){
	var vRow = this.getRowFromIndex(pRow);
	if(!vRow) return false;
	return vRow.isContainer();
}
ItemListTreeView.prototype.isContainerOpen = function(pRow){
	var vRow = this.getRowFromIndex(pRow);
	if(!vRow) return false;
	return vRow.isContainerOpen();
}
ItemListTreeView.prototype.isContainerEmpty = function(pRow){
	var vRow = this.getRowFromIndex(pRow);
	if(!vRow) return false;
	return vRow.isContainerEmpty();
}
ItemListTreeView.prototype.toggleOpenState = function(pRow){
	var vRow = this.getRowFromIndex(pRow);
	if(vRow) vRow.toggleOpenState(pRow);
}



ItemListTreeView.prototype.canChangeSelect = function(){
	return true;
}







ItemListTreeView.prototype.getProgressMode = function (pRow, pCol){
	return this.getCellBuiler(pCol).getProgressMode(this.getRowFromIndex(pRow), pCol);
}
ItemListTreeView.prototype.getCellValue = function(pRow, pCol){
	return this.getCellBuiler(pCol).getCellValue(this.getRowFromIndex(pRow), pCol);
}
ItemListTreeView.prototype.cycleCell = function(pRow, pCol){
	this.getCellBuiler(pCol).cycleCell(this.getRowFromIndex(pRow), pCol);
}
ItemListTreeView.prototype.cycleHeader = function(pCol){
	this.getCellBuiler(pCol).cycleHeader(this, pCol);
}





ItemListTreeView.prototype.canDrop = function(pRow, pOrientation){
	return false;
}
ItemListTreeView.prototype.drop = function(pRow, pOrientation){

} 



ItemListTreeView.prototype.selectionChanged = function(){	

}
ItemListTreeView.prototype.performAction = function(pAction){

}
ItemListTreeView.prototype.performActionOnRow = function(pAction, pRow){

}
ItemListTreeView.prototype.performActionOnCell = function(pAction, pRow, pColId){

}
ItemListTreeView.prototype.setCellText = function(pRow, pColId, pValue){

}
ItemListTreeView.prototype.setCellValue = function(pRow, pColId, pValue){

}




ItemListTreeView.prototype.getRowFromIndex = function(pIndex){
	if(this.fLastIdx == pIndex) return this.fLastRow;
	var vOffset = pIndex;
	for(var i = 0; i < this.fCh.length; i++) {
		var vC = this.fCh[i].rowCount;
		if(vC > vOffset) {
			this.fLastIdx = pIndex;
			return this.fLastRow = this.fCh[i].xGetRow(vOffset);
		}
		vOffset -= vC;
	}
	return null;
}
ItemListTreeView.prototype.getXulTree = function(){
	return this.fXulTree;
}



ItemListTreeView.prototype.getIndex = function(){
	return -1;
}
ItemListTreeView.prototype.xDeltaChildren = function(pDelta){
	this.rowCount += pDelta;
	this.fLastIdx = -1;
	this.fLastRow = null;
}
ItemListTreeView.prototype.getItemListTreeView = function(){
	return this;
}




function ILRowFolder(pParent, pOffsetInParent, pDatas, pChildren, pIsOpen){
	this.fParent = pParent;
	this.fOffsetInParent = pOffsetInParent;
	this.fCh = pChildren || [];
	this.fDatas = pDatas;
	this.fIsOpen = pIsOpen;
	this.rowCount = 1;
	if(pIsOpen) for(var i=0; i < this.fCh.length; i++) this.rowCount += pChildren.rowCount;
}
ILRowFolder.prototype.getItemListTreeView = function(){
	return this.fParent.getItemListTreeView();
}

ILRowFolder.prototype.getXulTree = function(){
	return this.fParent.getXulTree();
}

ILRowFolder.prototype.getIndex = function(){
	var vOffsetRow = this.fParent.getIndex() + 1;
	for(var i = 0; i < this.fOffsetInParent; i++) {
		vOffsetRow += this.fParent.fCh[i].rowCount;
	}
	return vOffsetRow;
}
ILRowFolder.prototype.isContainer = function(){
	return true;
}
ILRowFolder.prototype.isContainerOpen = function(){
	return this.fIsOpen;
}
ILRowFolder.prototype.isContainerEmpty = function(){
	return this.fCh.length==0;
}
ILRowFolder.prototype.toggleOpenState = function(pIdxRow){
	var vTreeBo = this.fParent.getXulTree().treeBoxObject;
	var vDelta;
	if(this.fIsOpen) {
		vDelta = 1 - this.rowCount;
	} else {
		vDelta = 0;
		for(var i=0; i < this.fCh; i++) vDelta += this.fCh[i].rowCount;
	}
	this.xDeltaChildren(vDelta);
	vTreeBo.rowCountChanged(pIdxRow+1, vDelta);
	vTreeBo.invalidateRow(pIdxRow);
	this.fIsOpen = ! this.fIsOpen;
}



ILRowFolder.prototype.xDeltaChildren = function(pDelta){
	this.rowCount += pDelta;
	this.fParent.xDeltaChildren(pDelta);
}

ILRowFolder.prototype.xGetRow = function(pOffset){
	if(pOffset==0) return this;
	if( ! this.fIsOpen) return null;
	pOffset -= 1;
	for(var i = 0; i < this.fCh.length; i++) {
		var vC = this.fCh[i].rowCount;
		if(vC > pOffset) return this.fCh[i].xGetRow(pOffset);
		pOffset -= vC;
	}
	return null;
}





function ILRowLeaf(pParent, pOffsetInParent, pDatas){
	this.fParent = pParent;
	this.fOffsetInParent = pOffsetInParent;
	this.fDatas = pDatas;
}
ILRowLeaf.prototype.__defineGetter__("rowCount", function(){return 1});
ILRowLeaf.prototype.getCellBuilder = ILRowFolder.prototype.getCellBuilder;
ILRowLeaf.prototype.getXulTree = ILRowFolder.prototype.getXulTree;
ILRowLeaf.prototype.getIndex = ILRowFolder.prototype.getIndex;
ILRowLeaf.prototype.isContainer = ILRowLeaf.prototype.isContainerOpen = ILRowLeaf.prototype.isContainerEmpty = function(){
	return false;
}
ILRowLeaf.prototype.toggleOpenState = function(pIdxRow){
}
ILRowLeaf.prototype.xGetRow = function(pOffset){
	return (pOffset==0) ? this : null;
}





function CellBuilderBase(){
}
CellBuilderBase.prototype.getCellText = function(pILRow, pCol) {
	return "";
}
CellBuilderBase.prototype.getCellValue = CellBuilderBase.prototype.getCellText;
CellBuilderBase.prototype.getProgressMode = function(pILRow, pCol) {
	return 3;
}
CellBuilderBase.prototype.getImageSrc = function(pILRow, pCol) {
	return null;
}
CellBuilderBase.prototype.getRowProperties = function(pILRow, pProps) {}
CellBuilderBase.prototype.getCellProperties = function(pILRow, pCol, pProps) {}
CellBuilderBase.prototype.getColumnProperties = function(pILTreeView, pCol, pProps) {}
CellBuilderBase.prototype.cycleCell = function(pILRow, pCol) {}
CellBuilderBase.prototype.cycleHeader = function(pILTreeView, pCol) {}

CellBuilderBase.prototype.buildTree = function(pILTreeView, pPrimaryCol) {
	pILTreeView.defaultBuildTree();
}

CellBuilderBase.prototype.sortPoolDatas = function(pILTreeView, pSortedCol) {
	throw "sortPoolDatas not implemented";
}




function CellBuilderLeafString(pDataOffset, pSortable){
	this.fDataOffset = pDataOffset;
	if(pSortable) this.fSortable = true;
}
CellBuilderLeafString.prototype = new CellBuilderBase();
CellBuilderLeafString.prototype.getCellText = function(pILRow, pCol) {
	if(!pILRow || pILRow.fCh) return "";
	return pILRow.fDatas[this.fDataOffset];
}
CellBuilderLeafString.prototype.cycleHeader = function(pILTreeView, pCol) {
	if(this.fSortable) {
		var vDescOrder = pCol.element.getAttribute("sortDirection")=="ascending";
		pILTreeView.setSortedColumn(pCol,  vDescOrder ? "descending" : "ascending");
	}
}
CellBuilderLeafString.prototype.sortPoolDatas = function(pILTreeView, pSortedCol) {
	var vDatas = pILTreeView.getPoolDatas();
	var vDataOffset = this.fDataOffset;
	if(pSortedCol.element.getAttribute("sortDirection")=="descending") {
		vDatas.sort(function(p1, p2) {
			if(p1[vDataOffset]==null) {
				if(p2[vDataOffset]==null) return 0;
				return 1;
			} else if(p2[vDataOffset]==null) {
				return -1;
			}
			return p2[vDataOffset].localeCompare(p1[vDataOffset]);
		});
	} else {
		vDatas.sort(function(p1, p2) {
			if(p1[vDataOffset]==null) {
				if(p2[vDataOffset]==null) return 0;
				return -1;
			} else if(p2[vDataOffset]==null) {
				return 1;
			}
			return p1[vDataOffset].localeCompare(p2[vDataOffset]);
		});
	}
}



function CellBuilderLeafInt(pDataOffset, pSortable){
	this.fDataOffset = pDataOffset;
	if(pSortable) this.fSortable = true;
}
CellBuilderLeafInt.prototype = new CellBuilderBase();
CellBuilderLeafInt.prototype.getCellText = CellBuilderLeafString.prototype.getCellText;
CellBuilderLeafInt.prototype.cycleHeader = CellBuilderLeafString.prototype.cycleHeader;
CellBuilderLeafInt.prototype.sortPoolDatas = function(pILTreeView, pSortedCol) {
	var vDatas = pILTreeView.getPoolDatas();
	var vDataOffset = this.fDataOffset;
	if(pSortedCol.element.getAttribute("sortDirection")=="descending") {
		vDatas.sort(function(p1, p2) {
			if(p1[vDataOffset]==null) {
				if(p2[vDataOffset]==null) return 0;
				return 1;
			} else if(p2[vDataOffset]==null) {
				return -1;
			}
			return p2[vDataOffset]-p1[vDataOffset];
		});
	} else {
		vDatas.sort(function(p1, p2) {
			if(p1[vDataOffset]==null) {
				if(p2[vDataOffset]==null) return 0;
				return -1;
			} else if(p2[vDataOffset]==null) {
				return 1;
			}
			return p1[vDataOffset]-p2[vDataOffset];
		});
	}
}


function CellBuilderLeafDate(pDataOffset, pSortable){
	this.fDataOffset = pDataOffset;
	if(pSortable) this.fSortable = true;
}
CellBuilderLeafDate.prototype = new CellBuilderBase();
CellBuilderLeafDate.prototype.getCellText = function(pILRow, pCol) {
	if(!pILRow || pILRow.fCh) return "";
	return new Date(pILRow.fDatas[this.fDataOffset]).toLocaleString();
}
CellBuilderLeafDate.prototype.cycleHeader = CellBuilderLeafString.prototype.cycleHeader;
CellBuilderLeafDate.prototype.sortPoolDatas = CellBuilderLeafInt.prototype.sortPoolDatas;

